@@ -417,9 +417,14 @@ QThemeIconInfo XdgIconLoader::findIconHelper(const QString &themeName,
             // Try to reduce the amount of subDirs by looking in the GTK+ cache in order to save
             // a massive amount of file stat (especially if the icon is not there)
             auto cache = theme.m_gtkCaches.at(i);
+            bool isFindInCache = false; // 是否从缓存中查找图标
             if (cache->isValid() || cache->reValid(true)) {
                 const auto result = cache->lookup(iconNameFallback);
-                if (cache->isValid()) {
+                // 如果再次确认缓存可用
+                if (cache->isValid() &&
+                    // 如果图标对应的缓存子目录列表不为空
+                    !result.isEmpty()) {
+                    isFindInCache = true;
                     const QVector<QIconDirInfo> subDirsCopy = subDirs;
                     subDirs.clear();
                     subDirs.reserve(result.count());
@@ -435,36 +440,47 @@ QThemeIconInfo XdgIconLoader::findIconHelper(const QString &themeName,
                 }
             }
 
-            QString contentDir = contentDirs.at(i) + QLatin1Char('/');
-            for (int j = 0; j < subDirs.size() ; ++j) {
-                const QIconDirInfo &dirInfo = subDirs.at(j);
-                const QString subDir = contentDir + dirInfo.path + QLatin1Char('/');
-                const QString pngPath = subDir + pngIconName;
-                if (QFile::exists(pngPath)) {
-                    PixmapEntry *iconEntry = new PixmapEntry;
-                    iconEntry->dir = dirInfo;
-                    iconEntry->filename = pngPath;
-                    // Notice we ensure that pixmap entries always come before
-                    // scalable to preserve search order afterwards
-                    info.entries.prepend(iconEntry);
-                } else {
-                    const QString svgPath = subDir + svgIconName;
-                    if (gSupportsSvg && QFile::exists(svgPath)) {
-                        ScalableEntry *iconEntry = (followColorScheme() && theme.followsColorScheme()) ? new ScalableFollowsColorEntry : new ScalableEntry;
+            // 从子目录中获取图标
+            auto getEntriesFromSubDirs = [=](QThemeIconEntries &entries, const QString &contentDir
+                    , const QVector<QIconDirInfo> &subDirs) {
+                for (int j = 0; j < subDirs.size(); ++j) {
+                    const QIconDirInfo &dirInfo = subDirs.at(j);
+                    const QString subDir = contentDir + dirInfo.path + QLatin1Char('/');
+                    const QString pngPath = subDir + pngIconName;
+                    if (QFile::exists(pngPath)) {
+                        PixmapEntry *iconEntry = new PixmapEntry;
                         iconEntry->dir = dirInfo;
-                        iconEntry->filename = svgPath;
-                        info.entries.append(iconEntry);
+                        iconEntry->filename = pngPath;
+                        // Notice we ensure that pixmap entries always come before
+                        // scalable to preserve search order afterwards
+                        entries.prepend(iconEntry);
+                    } else {
+                        const QString svgPath = subDir + svgIconName;
+                        if (gSupportsSvg && QFile::exists(svgPath)) {
+                            ScalableEntry *iconEntry = (followColorScheme() && theme.followsColorScheme()) ? new ScalableFollowsColorEntry : new ScalableEntry;
+                            iconEntry->dir = dirInfo;
+                            iconEntry->filename = svgPath;
+                            entries.append(iconEntry);
+                        }
+                    }
+                    const QString xpmPath = subDir + xpmIconName;
+                    if (QFile::exists(xpmPath)) {
+                        PixmapEntry *iconEntry = new PixmapEntry;
+                        iconEntry->dir = dirInfo;
+                        iconEntry->filename = xpmPath;
+                        // Notice we ensure that pixmap entries always come before
+                        // scalable to preserve search order afterwards
+                        entries.append(iconEntry);
                     }
                 }
-                const QString xpmPath = subDir + xpmIconName;
-                if (QFile::exists(xpmPath)) {
-                    PixmapEntry *iconEntry = new PixmapEntry;
-                    iconEntry->dir = dirInfo;
-                    iconEntry->filename = xpmPath;
-                    // Notice we ensure that pixmap entries always come before
-                    // scalable to preserve search order afterwards
-                    info.entries.append(iconEntry);
-                }
+            };
+
+            QString contentDir = contentDirs.at(i) + QLatin1Char('/');
+            getEntriesFromSubDirs(info.entries, contentDir, subDirs);
+            // 如果从缓存中没有找到图标，则在主题全目录进行查找
+            if (info.entries.isEmpty() && isFindInCache) {
+                subDirs = theme.keyList();
+                getEntriesFromSubDirs(info.entries, contentDir, subDirs);
             }
         }
 
